<section id="settings">
  <h1>
    <a href="/">⬅</a>
    Settings
  </h1>
  
  {% if updated %}
  <p class="alert">
    <code>{{updated}}</code> 
    ✅ Settings Updated!
  </p>
  {% endif %}
  
  <form action="/save_settings" method="post">
    <label>
      <small>OPENAI API KEY*</small>
      <input type="text" name="api_key" value="{{current['api_key']}}" required>
    </label>
    <label>
      <small>MODEL NAME*</small>
      <input type="text" name="api_model" value="{{current['api_model']}}" required>
    </label>
    <label>
      <small>OPENAI API ENDPOINT (Leave blank to use default)</small>
      <input type="text" name="api_endpoint" value="{{current['api_endpoint']}}">
    </label>
    <button>💾 Save</button>
  </form>
</section>
