<section id="setup-prompt">
  <h1>Are You Ready? 🔥</h1>
  <p>We need to set you up though.</p>
  <p>Head over to <a href="/settings">settings</a> and add your LLM API Key.</p>
  <p><a href="/settings" class="btn">⚙ Open Settings</a>
</section>
